﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ExtendedShoppingCartController.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   Extended shopping cart controller class
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.Linq;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using Microsoft.Dynamics.Commerce.Runtime;
    using Microsoft.Dynamics.Commerce.Runtime.Client;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Controllers;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Mappers;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using CrtDataModel = Microsoft.Dynamics.Commerce.Runtime.DataModel;

    /// <summary>
    /// Extended checkout controller class
    /// </summary>
    public class ExtendedShoppingCartController : ShoppingCartController
    {
        /// <summary>
        /// Deletes the carts.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="cartIds">The cart ids.</param>
        public void DeleteCarts(string customerId, IEnumerable<string> cartIds)
        {
            if (cartIds == null)
            {
                throw new ArgumentNullException("cartIds");
            }

            OrderManager orderManager = OrderManager.Create(CrtUtilities.GetCommerceRuntime());
            if (orderManager == null)
            {
                throw new InvalidOperationException("OrderManager can not be null when updating a shipping method");
            }

            orderManager.DeleteCarts(customerId, cartIds);
        }

        /// <summary>
        /// Populates the cart item with kit details.
        /// </summary>
        /// <param name="cart">The shopping cart.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <returns>A shopping cart</returns>
        public virtual ShoppingCart AddKitDetailsToShoppingCart(CrtDataModel.Cart cart, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            CommerceRuntime runtime = CrtUtilities.GetCommerceRuntime();
            OrderManager manager = OrderManager.Create(runtime);

            // Get the product details for all the products in shopping cart.
            ReadOnlyCollection<CrtDataModel.Product> products = manager.GetProductsInCart(cart.Id);

            var mapper = new ExtendedShoppingCartMapper(this.Configuration);
            ShoppingCart shoppingCart = mapper.ConvertToViewModel(cart, ShoppingCartDataLevel.All, productValidator, searchEngine);
            var kitsMapper = new KitMapper(this.Configuration);
            kitsMapper.PopulateKitItemDetails(shoppingCart.Items, products, searchEngine);

            return shoppingCart;
        }

        /// <summary>
        /// Moves the items between carts.
        /// </summary>
        /// <param name="sourceShoppingCartId">The source shopping cart identifier.</param>
        /// <param name="destinationShoppingCartId">The destination shopping cart identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <returns>A shopping cart</returns>
        /// <exception cref="ArgumentNullException">
        /// sourceShoppingCartId
        /// or
        /// destinationShoppingCartId
        /// </exception>
        public override ShoppingCart MoveItemsBetweenCarts(string sourceShoppingCartId, string destinationShoppingCartId, string customerId, ShoppingCartDataLevel dataLevel, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (string.IsNullOrWhiteSpace(sourceShoppingCartId))
            {
                throw new ArgumentNullException("sourceShoppingCartId");
            }

            if (string.IsNullOrWhiteSpace(destinationShoppingCartId))
            {
                throw new ArgumentNullException("destinationShoppingCartId");
            }

            CrtDataModel.Cart anonymousCart = OrderManager.Create(CrtUtilities.GetCommerceRuntime()).GetCart(sourceShoppingCartId, null, CrtDataModel.CalculationModes.All);
            if (anonymousCart == null || !anonymousCart.CartLines.Any())
            {
                return this.GetShoppingCart(destinationShoppingCartId, customerId, dataLevel, productValidator, searchEngine);
            }

            ICollection<Listing> listingsFromCart = new ExtendedShoppingCartMapper(this.Configuration).GetListingsFromCart(anonymousCart);
            ShoppingCart shoppingCart = this.AddItems(destinationShoppingCartId, customerId, listingsFromCart, dataLevel, productValidator, searchEngine);
            if (anonymousCart.DiscountCodes.Any())
            {
                shoppingCart = anonymousCart.DiscountCodes.Aggregate(shoppingCart, (current, discountCode) => this.AddOrRemovePromotionCode(current.CartId, customerId, discountCode, true, dataLevel, productValidator, searchEngine));
            }
            
            IEnumerable<string> lineIds = anonymousCart.CartLines.Select(i => i.LineId);
            this.RemoveItems(sourceShoppingCartId, customerId, lineIds, dataLevel, productValidator, searchEngine);
            
            return shoppingCart;
        }

        /// <summary>
        /// Claims the anonymous cart.
        /// </summary>
        /// <param name="shoppingCartIdToBeClaimed">The shopping cart identifier to be claimed.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="dataLevel">The data level.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <returns></returns>
        /// <exception cref="DataValidationException">Shopping cart {0} of customer {1} was not found.;new object[2]
        ///         {
        ///           (object) shoppingCartIdToBeClaimed,
        ///           (object) customerId
        ///         }</exception>
        public override ShoppingCart ClaimAnonymousCart(string shoppingCartIdToBeClaimed, string customerId, ShoppingCartDataLevel dataLevel, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (string.IsNullOrWhiteSpace(shoppingCartIdToBeClaimed))
            {
                throw new ArgumentNullException("shoppingCartIdToBeClaimed");
            }

            if (string.IsNullOrWhiteSpace(customerId))
            {
                throw new ArgumentNullException("customerId");
            }

            CrtDataModel.Cart anonymousCart = OrderManager.Create(CrtUtilities.GetCommerceRuntime()).GetCart(shoppingCartIdToBeClaimed, null, CrtDataModel.CalculationModes.All);
            if (anonymousCart == null || !string.IsNullOrWhiteSpace(anonymousCart.CustomerId))
            {
                return null;
            }

            var mapper = new ExtendedShoppingCartMapper(this.Configuration);
            var listingsFromCart = mapper.GetListingsFromCart(anonymousCart);
            CrtDataModel.Cart authenticatedCart = new CrtDataModel.Cart { Id = shoppingCartIdToBeClaimed, CustomerId = customerId, CartType = CrtDataModel.CartType.Shopping };
            ShoppingCart authenticatedShoppingCart = mapper.ConvertToViewModel(authenticatedCart, dataLevel, productValidator, searchEngine);
            if (authenticatedCart == null && authenticatedShoppingCart == null)
            {
                throw new DataValidationException(DataValidationErrors.CartNotFound, "Shopping cart {0} of customer {1} was not found.", shoppingCartIdToBeClaimed, customerId);
            }

            authenticatedShoppingCart = this.AddItems(authenticatedShoppingCart.CartId, customerId, listingsFromCart, dataLevel, productValidator, searchEngine);
            if (anonymousCart.DiscountCodes.Any())
            {
                authenticatedShoppingCart = anonymousCart.DiscountCodes.Aggregate(authenticatedShoppingCart, (current, discountCode) => this.AddOrRemovePromotionCode(current.CartId, customerId, discountCode, true, dataLevel, productValidator, searchEngine));
            }

            IEnumerable<string> lineIds = anonymousCart.CartLines.Select(i => i.LineId);
            this.RemoveItems(shoppingCartIdToBeClaimed, customerId, lineIds, dataLevel, productValidator, searchEngine);
            
            return authenticatedShoppingCart;
        }
    }
}