﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="IExtendedShoppingCartService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   The extended shopping cart service interface.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System.Collections.Generic;
    using System.ServiceModel;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;

    /// <summary>
    /// The extended shopping cart service interface.
    /// </summary>
    [ServiceContract]
    public interface IExtendedShoppingCartService
    {
        /// <summary>
        /// Loads a shopping cart.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id</param>
        /// <param name="customerId">The customer id</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the updated shopping cart.</returns>
        [OperationContract]
        ShoppingCartResponse GetShoppingCartExtended(string shoppingCartId, string customerId, ShoppingCartDataLevel dataLevel);

        /// <summary>
        /// Adds items to the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="listings">The items to add to the cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the updated shopping cart.</returns>
        [OperationContract]
        ShoppingCartResponse AddItemsExtended(string shoppingCartId, string customerId, IEnumerable<Listing> listings, ShoppingCartDataLevel dataLevel);
        
        /// <summary>
        /// Removes items from the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="lineIds">The item line identifiers to be removed from the shopping cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the updated shopping cart.</returns>
        [OperationContract]
        ShoppingCartResponse RemoveItemsExtended(string shoppingCartId, string customerId, IEnumerable<string> lineIds, ShoppingCartDataLevel dataLevel);

        /// <summary>
        /// Updates items on the shopping cart associated with the given identifier.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="items">The items to be updated.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the updated shopping cart.</returns>
        [OperationContract]
        ShoppingCartResponse UpdateItemsExtended(string shoppingCartId, string customerId, IEnumerable<TransactionItem> items, ShoppingCartDataLevel dataLevel);
        
        /// <summary>
        /// Merges anonymous and active shopping carts for the current user.
        /// </summary>
        /// <param name="anonymousShoppingCartId">The anonymous shopping cart id</param>
        /// <param name="customerId">The customer id</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The service response containing the merged shopping cart.</returns>
        [OperationContract]
        ShoppingCartResponse MergeCarts(string anonymousShoppingCartId, string customerId, ShoppingCartDataLevel dataLevel);

        /// <summary>
        /// Gets all the promotions for the shopping cart items.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>
        /// The service response containing the updated shopping cart.
        /// </returns>
        [OperationContract]
        ShoppingCartResponse GetPromotionsExtended(string shoppingCartId, string customerId, ShoppingCartDataLevel dataLevel);

        /// <summary>
        /// Adds or Removes the discount codes from the cart.
        /// </summary>
        /// <param name="shoppingCartId">The shopping cart id.</param>
        /// <param name="customerId">The customer id.</param>
        /// <param name="promotionCode">The promotion code to apply to the cart.</param>
        /// <param name="isAdd">Indicates whether the operation is addition or removal of discount codes.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>The shopping cart.</returns>
        [OperationContract]
        ShoppingCartResponse AddOrRemovePromotionCodeExtended(string shoppingCartId, string customerId, string promotionCode, bool isAdd, ShoppingCartDataLevel dataLevel);
    }
}
