﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ExtendedShoppingCartMapper.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   Extended shopping cart mapper class
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;
    using Microsoft.Dynamics.Commerce.Runtime.Client;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using CrtDataModel = Microsoft.Dynamics.Commerce.Runtime.DataModel;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Mappers;

    /// <summary>
    /// Extended shopping cart mapper class
    /// </summary>
    public class ExtendedShoppingCartMapper
    {
        private readonly IConfiguration _configurationValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtendedShoppingCartMapper"/> class.
        /// </summary>
        /// <param name="configuration">The configuration.</param>
        public ExtendedShoppingCartMapper(IConfiguration configuration)
        {
            this._configurationValue = configuration;
            // From a perf view, it might be better to cache the gift card item id, instead of fetching the channel configuration everytime.
            ChannelManager channelManager = ChannelManager.Create(CrtUtilities.GetCommerceRuntime());
            this.GiftCardItemId = channelManager.GetChannelConfiguration().GiftCardItemId;
        }

        /// <summary>
        /// Gets or sets the gift card item identifier.
        /// </summary>
        /// <value>
        /// The gift card item identifier.
        /// </value>
        public string GiftCardItemId { get; set; }

        /// <summary>
        /// Converts from CRT Cart to view model ShoppingCart.
        /// </summary>
        /// <param name="shoppingCart">The CRT shopping cart.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <returns>A view model Shopping Cart.</returns>
        public ShoppingCart ConvertToViewModel(CrtDataModel.Cart shoppingCart, ShoppingCartDataLevel dataLevel, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (shoppingCart == null)
            {
                return null;
            }

            if (!string.IsNullOrWhiteSpace(shoppingCart.CustomerId))
            {
                CustomerManager.Create(CrtUtilities.GetCommerceRuntime()).GetCustomer(shoppingCart.CustomerId);
            }

            ShoppingCart cart = new ShoppingCart
            {
                CartId = shoppingCart.Id,
                TotalAmount = shoppingCart.TotalAmount,
                TotalAmountWithCurrency = shoppingCart.TotalAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture()),
                LastModifiedDate = shoppingCart.ModifiedDateTime.HasValue ? shoppingCart.ModifiedDateTime.Value.DateTime : DateTime.UtcNow
            };

            if (dataLevel > ShoppingCartDataLevel.Minimal)
            {
                cart.DiscountAmountWithCurrency = shoppingCart.DiscountAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                cart.DiscountAmount = shoppingCart.DiscountAmount;
                cart.ChargeAmountWithCurrency = shoppingCart.ChargeAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                cart.SubtotalWithCurrency = shoppingCart.SubtotalAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                cart.DeliveryModeId = shoppingCart.DeliveryMode;
                cart.ShippingAddress = ExtendedAddressMapper.ConvertToViewModel(shoppingCart.ShippingAddress);
                cart.CartType = (CartType)shoppingCart.CartType;
                cart.Name = shoppingCart.Name;
                cart.LoyaltyCardId = shoppingCart.LoyaltyCardId;
                cart.TaxAmountWithCurrency = shoppingCart.TaxAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                if (shoppingCart.DiscountCodes != null)
                {
                    foreach (string str in shoppingCart.DiscountCodes)
                    {
                        cart.DiscountCodes.Add(str);
                    }
                }
            }

            if (dataLevel == ShoppingCartDataLevel.All)
            {
                foreach (string str in shoppingCart.PromotionLines)
                {
                    cart.PromotionLines.Add(str);
                }
            }

            Collection<TransactionItem> transactionItems = this.ConvertToViewModel(shoppingCart.CartLines, dataLevel, productValidator, searchEngine);
            if (transactionItems != null)
            {
                foreach (TransactionItem transactionItem in transactionItems)
                {
                    cart.Items.Add(transactionItem);
                }
            }

            return cart;
        }

        /// <summary>
        /// Converts CRT cart lines to view model shopping cart items.
        /// </summary>
        /// <param name="cartLines">The CRT cart lines.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>A view model collection of shopping cart items.</returns>
        public Collection<TransactionItem> ConvertToViewModel(IEnumerable<CrtDataModel.CartLine> cartLines, ShoppingCartDataLevel dataLevel, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (cartLines == null)
            {
                return null;
            }

            var cartItems = new Collection<TransactionItem>();

            const bool isResultManadatory = false;
            IDictionary<long, IDictionary<string, object>> propertiesByProductId = searchEngine.GetListingsProperties(
                isResultManadatory,
                cartLines.Select(cl => cl.ProductId),
                searchEngine.ImageSearchPropertyName,
                searchEngine.PathSearchPropertyName
                );

            foreach (CrtDataModel.CartLine cartLine in cartLines)
            {
                IDictionary<string, object> productProperties;
                propertiesByProductId.TryGetValue(cartLine.ProductId, out productProperties);
                cartItems.Add(this.ConvertToViewModel(cartLine, dataLevel, productProperties, productValidator, searchEngine));
            }

            return cartItems;
        }

        /// <summary>
        /// Converts CRT cart line to view model shopping cart item.
        /// </summary>
        /// <param name="cartLine">The CRT cart line.</param>
        /// <param name="properties">The key value pairs of product properties.</param>
        /// <param name="productValidator">The product validator.</param>
        /// <param name="searchEngine">The search engine.</param>
        /// <param name="dataLevel">Represents the data level of the cart entity that is required by the caller.</param>
        /// <returns>A view model shopping cart item.</returns>
        public TransactionItem ConvertToViewModel(CrtDataModel.CartLine cartLine, ShoppingCartDataLevel dataLevel, IDictionary<string, object> properties, IProductValidator productValidator, ISearchEngine searchEngine)
        {
            if (cartLine == null)
            {
                return null;
            }

            var cartItem = new TransactionItem
            {
                LineId = cartLine.LineId,
                ItemId = cartLine.LineData.ItemId,
                VariantInventoryDimensionId = cartLine.LineData.InventoryDimensionId,
                ProductId = cartLine.LineData.ProductId,
                ProductDetails = (string) cartLine.LineData["ProductDetails"],
                Quantity = (int) cartLine.LineData.Quantity,
                NetAmountWithCurrency = (cartLine.LineData.TotalAmount - cartLine.LineData.TaxAmountExclusive).ToCurrencyString(this._configurationValue.GetCurrentCulture()),
                Comment = cartLine.Comment,
                Image = new ImageInfo(),
                ProductUrl = Utilities.InactiveLinkUrl
            };

            if (properties != null)
            {
                object imageSearchValue;
                if (properties.TryGetValue(searchEngine.ImageSearchPropertyName, out imageSearchValue))
                {
                    cartItem.Image = (ImageInfo)imageSearchValue;
                }

                object pathValue;
                if (properties.TryGetValue(searchEngine.PathSearchPropertyName, out pathValue))
                {
                    cartItem.ProductUrl = (string)pathValue;
                }
            }

            if (dataLevel > ShoppingCartDataLevel.Minimal)
            {
                cartItem.TaxAmountWithCurrency = cartLine.LineData.TaxAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                cartItem.DiscountAmountWithCurrency = cartLine.LineData.DiscountAmount.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                cartItem.DiscountAmount = cartLine.LineData.DiscountAmount;
                cartItem.OfferNames = String.Join(",", cartLine.LineData.DiscountLines.Select(dl => dl.OfferName));
                cartItem.PriceWithCurrency = cartLine.LineData.Price.ToCurrencyString(this._configurationValue.GetCurrentCulture());
                cartItem.DeliveryModeId = cartLine.LineData.DeliveryMode;
                cartItem.ElectronicDeliveryEmail = cartLine.LineData.ElectronicDeliveryEmail;
                cartItem.ShippingAddress = ExtendedAddressMapper.ConvertToViewModel(cartLine.LineData.ShippingAddress);
            }

            if (dataLevel == ShoppingCartDataLevel.All)
            {
                foreach (var promotionLine in cartLine.PromotionLines)
                {
                    cartItem.PromotionLines.Add(promotionLine);
                }
            }

            // Throws an exception if the Product Details blob is malformed.
            productValidator.ValidateProductDetails(cartItem.ProductDetails);

            return cartItem;
        }

        /// <summary>
        /// Gets the listings from cart.
        /// </summary>
        /// <param name="shoppingCart">The shopping cart.</param>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException">shopping cart</exception>
        public ICollection<Listing> GetListingsFromCart(CrtDataModel.Cart shoppingCart)
        {
            if (shoppingCart == null)
            {
                throw new ArgumentNullException("shoppingCart");
            }

            ICollection<Listing> listings = new Collection<Listing>();
            foreach (CrtDataModel.CartLine cartLine in shoppingCart.CartLines)
            {
                Listing listing = new Listing
                        {
                            ListingId = cartLine.ProductId,
                            ProductDetails = (string) cartLine.LineData["ProductDetails"],
                            Quantity = (int) cartLine.Quantity,
                            Comment = cartLine.Comment
                        };
                
                if (cartLine.IsGiftCardLine)
                {
                    listing.GiftCardAmount = cartLine.Price;
                }

                listings.Add(listing);
            }

            return listings;
        }
    }
}