﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ChannelService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   The channel dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// Service for Channel.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class ChannelService : ChannelServiceBase, IExtendedChannelService
    {
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();
        
        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }
        
        #region EXTENSIONS

        /// <summary>
        /// Gets channel tender types.
        /// </summary>
        /// <returns>Tender types response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ExtendedTenderTypesResponse GetChannelTenderTypesExtended()
        {
            var response = new ExtendedTenderTypesResponse();
            this.ExecuteAfterRequestValidation(response, delegate
            {
                var tenderTypes = this.Controller.GetTenderTypes();
                response.SetTenderTypes(tenderTypes);

                Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models.ChannelConfiguration channelInfo = this.Controller.GetChannelConfiguration();
                response.GiftCardItemId = channelInfo.GiftCardItemId;
            });

            return response;
        }

        #endregion
        
        /// <summary>
        /// Gets the channel configuration.
        /// </summary>
        /// <returns>
        /// A channel info response that contains the channel configuration.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override ChannelInfoResponse GetChannelConfiguration()
        {
            return base.GetChannelConfiguration();
        }

        /// <summary>
        /// Gets channel tender types.
        /// </summary>
        /// <returns>Tender types response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override TenderTypesResponse GetChannelTenderTypes()
        {
            return base.GetChannelTenderTypes();
        } 

        /// <summary>
        /// Gets the channel country region info.
        /// </summary>
        /// <returns>Country info response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override CountryInfoResponse GetChannelCountryRegionInfo()
        {
            return base.GetChannelCountryRegionInfo();
        }

        /// <summary>
        /// Gets the states/provinces data for the given country.
        /// </summary>
        /// <param name="countryCode">Country code.</param>
        /// <returns>The states/provinces for the given country.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override StateProvinceInfoResponse GetStateProvinceInfo(string countryCode)
        {
            if (!string.IsNullOrEmpty(countryCode))
            {
                return base.GetStateProvinceInfo(countryCode);
            }

            StateProvinceInfoResponse response = new StateProvinceInfoResponse();
            response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("countryCode")));
            return response;
        }

        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, System.Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteAfterRequestValidation" /> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }
    }
}
