﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="CustomerService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   The customer dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.Collections.Generic;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System.Collections.ObjectModel;
    using System.Linq;
    using System.Diagnostics.CodeAnalysis;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Controllers;

    /// <summary>
    /// Service for Customers.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class CustomerService : CustomerServiceBase, IExtendedCustomerService
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        #region EXTENSIONS

        /// <summary>
        /// Creates the controller.
        /// </summary>
        /// <returns>This controller</returns>
        protected override CustomerController CreateController()
        {
            return Activator.CreateInstance<ExtendedCustomerController>();
        }

        /// <summary>
        /// Gets the customer.
        /// </summary>
        /// <param name="accountNumber">The customer account number.</param>
        /// <returns>
        /// A customer response.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual CustomerResponse GetCustomerExtended(string accountNumber)
        {
            var response = new CustomerResponse();
            if (string.IsNullOrEmpty(accountNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("accountNumber")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                response.Customer = this.Controller.GetCustomer(accountNumber);
            });

            return response;
        }

        /// <summary>
        /// Creates a customer.
        /// </summary>
        /// <param name="email">The customer's e-mail address.</param>
        /// <param name="givenName">Name of the given.</param>
        /// <param name="surname">Name of the sur.</param>
        /// <param name="userId">The user identifier.</param>
        /// <returns>
        /// A string response that contains the customer account number.
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual CreateUserResponse CreateCustomerExtended(string email, string givenName, string surname, string userId)
        {
            var response = new CreateUserResponse();
            if (string.IsNullOrEmpty(email))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("email")));
            }

            if (string.IsNullOrEmpty(userId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("userId")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                this.Controller.SetAuthorizationManager(new AuthorizationManager());
                response.AccountNumber = this.Controller.AssociateCustomer(email, givenName, surname, userId);
            });

            return response;
        }

        /// <summary>
        /// Gets the customer addresses.
        /// </summary>
        /// <param name="accountNumber">The customer account number.</param>
        /// <returns>An address collection response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual AddressCollectionResponse GetAddressesExtended(string accountNumber)
        {
            var response = new AddressCollectionResponse();
            if (string.IsNullOrEmpty(accountNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("accountNumber")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            Collection<Address> addresses = null;
            this.ExecuteAfterRequestValidation(response, () =>
            {
                addresses = this.Controller.GetAddresses(accountNumber);
            });

            if ((addresses == null) || !addresses.Any())
            {
                addresses = new Collection<Address> { new Address() };
            }

            response.Addresses = addresses;
            return response;
        }

        /// <summary>
        /// Updates the customer addresses.
        /// </summary>
        /// <param name="accountNumber">The customer account number.</param>
        /// <param name="addresses">The addresses.</param>
        /// <returns>Call success/failure response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual NullResponse UpdateAddressesExtended(string accountNumber, IEnumerable<Address> addresses)
        {
            var response = new NullResponse();
            if (string.IsNullOrEmpty(accountNumber))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("accountNumber")));
            }

            var updatedAddresses = addresses as Address[] ?? addresses.ToArray();
            if (addresses == null || !updatedAddresses.ToList().Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("addresses")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            this.ExecuteAfterRequestValidation(response, () => this.Controller.UpdateAddresses(accountNumber, updatedAddresses));

            return response;
        }

        #endregion

        /// <summary>
        /// Gets the customer.
        /// </summary>
        /// <returns>Customer response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override CustomerResponse GetCustomer()
        {
            return base.GetCustomer();
        }

        /// <summary>
        /// Creates the customer.
        /// </summary>
        /// <param name="customer">The customer.</param>
        /// <returns>Customer response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override CustomerResponse CreateCustomer(Customer customer)
        {
            var response = new CustomerResponse();
            if (customer == null)
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("customer")));
            }

            return response.HasErrors ? response : base.CreateCustomer(customer);
        }

        /// <summary>
        /// Creates the customer.
        /// </summary>
        /// <param name="emailAddress">Email address of the customer.</param>
        /// <param name="activationToken">Activation token.</param>
        /// <returns>Customer response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override CustomerResponse CreateCustomerByEmail(string emailAddress, string activationToken)
        {
            var response = new CustomerResponse();
            if (string.IsNullOrEmpty(emailAddress))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("emailAddress")));
            }

            if (string.IsNullOrEmpty(activationToken))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("activationToken")));
            }

            return response.HasErrors ? response : base.CreateCustomerByEmail(emailAddress, activationToken);
        }

        /// <summary>
        /// Search for a customer given the corresponding email address.
        /// </summary>
        /// <param name="emailAddress">Email address of the customer.</param>
        /// <returns>True if the customer exists, false otherwise.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override bool SearchCustomers(string emailAddress)
        {
            return base.SearchCustomers(emailAddress);
        }

        /// <summary>
        /// Updates the customer.
        /// </summary>
        /// <param name="customer">The customer.</param>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public override NullResponse UpdateCustomer(Customer customer)
        {
            var response = new NullResponse();

            this.ExecuteAfterRequestValidation(response, () => this.Controller.UpdateCustomer(customer));

            return response;
        }

        /// <summary>
        /// Gets the addresses.
        /// </summary>
        /// <returns>An address collection response.</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services.AddressCollectionResponse GetAddresses()
        {
            return base.GetAddresses();
        }

        /// <summary>
        /// Updates the addresses.
        /// </summary>
        /// <param name="addresses">The addresses.</param>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override NullResponse UpdateAddresses(IEnumerable<Address> addresses)
        {
            var response = new NullResponse();
            var enumerable = addresses as Address[] ?? addresses.ToArray();
            if (addresses == null || !enumerable.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("addresses")));
            }

            return response.HasErrors ? response : base.UpdateAddresses(enumerable);
        }

        /// <summary>
        /// Associates an existing customer to the current credentials if the criteria matches.
        /// </summary>
        /// <param name="email">The customer's e-mail address.</param>
        /// <param name="givenName">Given name.</param>
        /// <param name="surname">Surname.</param>
        /// <param name="userId">The user identifier.</param>
        /// <exception cref="Microsoft.Dynamics.Commerce.Runtime.DataValidationException">Thrown when the customer cannot be found.</exception>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override StringResponse AssociateCustomer(string email, string givenName, string surname, string userId)
        {
            var response = new StringResponse();
            if (string.IsNullOrEmpty(email))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("email")));
            }

            if (string.IsNullOrEmpty(userId))
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("userId")));
            }

            return response.HasErrors ? response : base.AssociateCustomer(email, givenName, surname, userId);
        }

        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Action to execute.</param>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteAfterRequestValidation"/> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            string customerId = Utilities.GetCustomerIdFromCookie();

            ExecuteAfterRequestValidation(response, () => action(customerId));
        }
    }
}

