﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="PricingService.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   The pricing dynamics service.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using System.Linq;
    using System.ServiceModel.Activation;
    using System.ServiceModel.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Storefront;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// The pricing dynamics service.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class PricingService : PricingServiceBase, IExtendedPricingService
    {
        private static readonly IProductValidator productValidator = new ProductValidator();
        private static readonly ISearchEngine searchEngine = new SearchEngine();
        private static readonly ServiceUtilityBase serviceProxy = new ServiceUtility();

        /// <summary>
        /// Gets Demo implementation of ISearchEngine.
        /// </summary>
        protected override ISearchEngine SearchEngine
        {
            get { return searchEngine; }
        }

        /// <summary>
        /// Gets Demo implementation of IProductValidator.
        /// </summary>
        protected override IProductValidator ProductValidator
        {
            get { return productValidator; }
        }

        /// <summary>
        /// Gets Demo implementation of ServiceUtilityBase.
        /// </summary>
        protected override ServiceUtilityBase ServiceUtility
        {
            get { return serviceProxy; }
        }

        /// <summary>
        /// Gets Demo implementation of IConfiguration.
        /// </summary>
        protected override IConfiguration Configuration
        {
            get { return new ServiceConfiguration(); }
        }

        #region EXTENSIONS

        /// <summary>
        /// Gets the active listing price filtered.
        /// </summary>
        /// <param name="listingIds">The listing ids.</param>
        /// <returns>
        /// Response containing active listing prices but only for the specific ids
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ListingPriceResponse GetActiveListingPriceFiltered(IEnumerable<long> listingIds)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            if (response.HasErrors)
            {
                return response;
            }
            
            response = base.GetActiveListingPrice(productIds);
            if (response.HasErrors)
            {
                return response;
            }

            // FILTERING THE RESULTED LISTING PRICES TO ONLY THE IDS THAT WERE PASSED IN
            var filteredListignPrices = response.ListingPrices.Where(l => productIds.Any(lid => lid == l.ListingId));
            response.ListingPrices = filteredListignPrices;

            return response;
        }
        
        /// <summary>
        /// Gets the listing price.
        /// </summary>
        /// <param name="listingIds">The listing ids.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="date">The date.</param>
        /// <returns>
        /// Response containing listing prices based on customer and/or date
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ListingPriceResponse GetActiveListingPriceExtended(IEnumerable<long> listingIds, string customerId, DateTime? date)
        {
            return this.GetActiveListingPriceWithAffiliationsAppliedExtended(listingIds, null, customerId, date);
        }

        /// <summary>
        /// Gets the listing price per catalog.
        /// </summary>
        /// <param name="listingIds">The listing ids.</param>
        /// <param name="channelId">The channel identifier.</param>
        /// <param name="catalogId">The catalog identifier.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="date">The date.</param>
        /// <returns>
        /// Response containing listing prices per catalog based on customer and/or date
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ListingPriceResponse GetActiveListingPricePerCatalogExtended(IEnumerable<long> listingIds, long channelId, long catalogId, string customerId, DateTime? date)
        {
            return this.GetActiveListingPricePerCatalogWithAffiliationsAppliedExtended(listingIds, channelId, catalogId, null, customerId, date);
        }

        /// <summary>
        /// Gets the listing price with affiliations applied.
        /// </summary>
        /// <param name="listingIds">The listing ids.</param>
        /// <param name="affiliationLoyaltyTierIds">The affiliation loyalty tier ids.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="date">The date.</param>
        /// <returns>
        /// Response containing listing prices with affiliations applied based on customer and/or date
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ListingPriceResponse GetActiveListingPriceWithAffiliationsAppliedExtended(IEnumerable<long> listingIds, IEnumerable<long> affiliationLoyaltyTierIds, string customerId, DateTime? date)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            if (!date.HasValue || date.Value == DateTime.MinValue)
            {
                date = DateTime.UtcNow;
            }

            if (customerId == null)
            {
                customerId = this.ServiceUtility.GetCurrentCustomerId(false);
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                var listingPrices = this.Controller.GetActiveListingPrice(productIds, 0L, new long?(), date.Value, customerId, null);

                // FILTERING THE RESULTED LISTING PRICES TO ONLY THE IDS THAT WERE PASSED IN
                var filteredListignPrices = listingPrices.Where(l => productIds.Any(lid => lid == l.ListingId));
                response.ListingPrices = filteredListignPrices;
            });

            return response;
        }
        
        /// <summary>
        /// Gets the listing price per catalog with affiliations applied.
        /// </summary>
        /// <param name="listingIds">The listing ids.</param>
        /// <param name="channelId">The channel identifier.</param>
        /// <param name="catalogId">The catalog identifier.</param>
        /// <param name="affiliationLoyaltyTierIds">The affiliation loyalty tier ids.</param>
        /// <param name="customerId">The customer identifier.</param>
        /// <param name="date">The date.</param>
        /// <returns>
        /// Response containing listing prices per catalog and with affiliations applied based on customer and/or date
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        public virtual ListingPriceResponse GetActiveListingPricePerCatalogWithAffiliationsAppliedExtended(IEnumerable<long> listingIds, long channelId, long catalogId, IEnumerable<long> affiliationLoyaltyTierIds, string customerId, DateTime? date)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            if (response.HasErrors)
            {
                return response;
            }

            if (!date.HasValue || date.Value == DateTime.MinValue)
            {
                date = DateTime.UtcNow;
            }

            if (customerId == null)
            {
                customerId = this.ServiceUtility.GetCurrentCustomerId(false);
            }

            this.ExecuteAfterRequestValidation(response, () =>
            {
                var listingPrices = this.Controller.GetActiveListingPrice(productIds, channelId, catalogId, date.Value, customerId, affiliationLoyaltyTierIds);

                // FILTERING THE RESULTED LISTING PRICES TO ONLY THE IDS THAT WERE PASSED IN
                var filteredListignPrices = listingPrices.Where(l => productIds.Any(lid => lid == l.ListingId));
                response.ListingPrices = filteredListignPrices;
            });

            return response;
        }

        #endregion

        /// <summary>
        /// Gets items' prices for the current channel.
        /// </summary>
        /// <param name="listingIds">Listing identifiers.</param>
        /// <returns>Response containing active listing prices</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ListingPriceResponse GetActiveListingPrice(IEnumerable<long> listingIds)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            return response.HasErrors ? response : base.GetActiveListingPrice(productIds);
        }

        /// <summary>
        /// Gets items' price for the give channel and catalog.
        /// </summary>
        /// <param name="listingIds">Listing identifiers.</param>
        /// <param name="channelId">The channel identifier.</param>
        /// <param name="catalogId">The catalog identifier.</param>
        /// <returns>Response containing active listing prices</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ListingPriceResponse GetActiveListingPricePerCatalog(IEnumerable<long> listingIds, long channelId, long catalogId)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            return response.HasErrors ? response : base.GetActiveListingPricePerCatalog(productIds, channelId, catalogId);
        }

        /// <summary>
        /// Gets items' prices for the current channel taking into account affiliation loyalty tiers.
        /// </summary>
        /// <param name="listingIds">Listing identifiers.</param>
        /// <param name="affiliationLoyaltyTierIds">Affiliation loyalty tiers IDS.</param>
        /// <returns>
        /// Response containing active listing prices
        /// </returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ListingPriceResponse GetActiveListingPriceWithAffiliationsApplied(IEnumerable<long> listingIds, IEnumerable<long> affiliationLoyaltyTierIds)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            return response.HasErrors ? response : base.GetActiveListingPriceWithAffiliationsApplied(productIds, affiliationLoyaltyTierIds);
        }

        /// <summary>
        /// Gets items' price for the give channel and catalog taking into account affiliation loyalty tiers.
        /// </summary>
        /// <param name="listingIds">Listing identifiers.</param>
        /// <param name="channelId">The channel identifier.</param>
        /// <param name="catalogId">The catalog identifier.</param>
        /// <param name="affiliationLoyaltyTierIds">Affiliation loyalty tier IDs.</param>
        /// <returns>Response containing active listing prices</returns>
        [WebInvoke(ResponseFormat = WebMessageFormat.Json, RequestFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.WrappedRequest)]
        [ExcludeFromCodeCoverage]
        public override ListingPriceResponse GetActiveListingPricePerCatalogWithAffiliationsApplied(IEnumerable<long> listingIds, long channelId, long catalogId, IEnumerable<long> affiliationLoyaltyTierIds)
        {
            var response = new ListingPriceResponse();
            var productIds = listingIds as long[] ?? listingIds.ToArray();
            if (listingIds == null || !productIds.Any())
            {
                response.Errors.AddRange(Utilities.HandleException(new ArgumentNullException("listingIds")));
            }

            return response.HasErrors ? response : base.GetActiveListingPricePerCatalogWithAffiliationsApplied(productIds, channelId, catalogId, affiliationLoyaltyTierIds);
        }

        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        protected override void ExecuteAfterRequestValidation(ServiceResponse response, System.Action action)
        {
            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            var serviceUtility = ServiceUtility as ServiceUtility;
            if (serviceUtility != null)
            {
                serviceUtility.SetSslCertPolicy();
            }

            Utilities.ExecuteAndHandleExceptions(response, action);
        }

        /// <summary>
        /// Executes handler and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void Execute(ServiceResponse response, Action action)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Checks if user is authenticated and assosiated then calls <see cref="ExecuteAfterRequestValidation" /> with new execution handler.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">The action.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        protected override void ExecuteForAuthorizedUser(ServiceResponse response, Action<string> action)
        {
            throw new NotImplementedException();
        }
    }
}
