﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ServiceUtility.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   The service utility class.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using System;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Controllers;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// The service utility class.
    /// </summary>
    [ExcludeFromCodeCoverage]
    internal class ServiceUtility : ServiceUtilityBase
    {
        /// <summary>
        /// This is used to turn off or on SSL Cert Validation so it can be done in one place.
        /// Needs to convert to configuration so cert validation can be turned off in development
        /// where selfssl certs are often used.
        /// </summary>
        public void SetSslCertPolicy()
        {
            // Trust all certificates
            // Needed for SelfSSL
            // TODO remove for production
            System.Net.ServicePointManager.ServerCertificateValidationCallback = ((sender, certificate, chain, sslPolicyErrors) => true);
        }

        /// <summary>
        /// Gets the culture.
        /// </summary>
        /// <returns>The ui culture</returns>
        public override string GetCulture()
        {
            return Utilities.GetCultureIdFromCookie();
        }

        /// <summary>
        /// Gets the current customer identifier.
        /// </summary>
        /// <param name="isMandatory">if set to <c>true</c> [is mandatory].</param>
        /// <returns>The current customer id</returns>
        public override string GetCurrentCustomerId(bool isMandatory)
        {
            return Utilities.GetCustomerIdFromCookie();
        }

        /// <summary>
        /// Gets the name of the user.
        /// </summary>
        /// <returns>The current user name</returns>
        public override string GetUserName()
        {
            // THIS SHOULD BE PROPERLY IMPLEMENTED IF NEEDED
            return Guid.NewGuid().ToString();
        }

        /// <summary>
        /// Clears the checkout cart identifier in persistence storage.
        /// </summary>
        public override void ClearCheckoutCartIdInPersistenceStorage()
        {
            Utilities.ClearCheckoutCartIdResponseCookies(SessionType.All);
        }

        /// <summary>
        /// Sets the cart identifier in persistence storage.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <param name="cartType">Type of the cart.</param>
        /// <param name="cartId">The cart identifier.</param>
        public override void SetCartIdInPersistenceStorage(SessionType sessionType, CartType cartType, string cartId)
        {
            Utilities.SetCartIdInResponseCookie(sessionType, cartType, cartId);
        }

        /// <summary>
        /// Gets the type of the session.
        /// </summary>
        /// <returns>Teh current session type</returns>
        /// <returns>Teh current session type</returns>
        public override SessionType GetSessionType()
        {
            return Utilities.GetSessionType();
        }

        /// <summary>
        /// Gets the shopping cart identifier from persistence storage.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <returns></returns>
        public override string GetShoppingCartIdFromPersistenceStorage(SessionType sessionType)
        {
            return Utilities.GetShoppingCartIdFromCookie(sessionType);
        }

        /// <summary>
        /// Gets the checkout cart identifier from persistence storage.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <returns></returns>
        public override string GetCheckoutCartIdFromPersistenceStorage(SessionType sessionType)
        {
            return Utilities.GetCheckoutCartIdFromCookie(sessionType);
        }

        /// <summary>
        /// Clears the shopping cart identifier in persistence storage.
        /// </summary>
        public override void ClearShoppingCartIdInPersistenceStorage()
        {
            Utilities.ClearShoppingCartIdResponseCookies(SessionType.All);
        }

        /// <summary>
        /// Determines whether this instance is associated.
        /// </summary>
        /// <returns></returns>
        /// <exception cref="System.NotImplementedException"></exception>
        public override bool IsAssociated()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets the user identifier.
        /// </summary>
        /// <returns></returns>
        /// <exception cref="System.NotImplementedException"></exception>
        public override string GetUserIdentifier()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Adds the custom claims.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        public override void AddCustomClaims(string customerId)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets the user email.
        /// </summary>
        /// <returns></returns>
        /// <exception cref="System.NotImplementedException"></exception>
        public override string GetUserEmail()
        {
            var customerId = Utilities.GetCustomerIdFromCookie();
            if (string.IsNullOrEmpty(customerId))
            {
                return string.Empty;
            }

            var customerController = new CustomerController();
            customerController.SetConfiguration(new ServiceConfiguration());
            var customer = customerController.GetCustomer(customerId);

            return customer.Email;
        }
    }
}