﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Utilities.cs" company="Sitecore Corporation">
//   Copyright (c) Sitecore Corporation 1999-2015
// </copyright>
// <summary>
//   The Utilities class.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Sitecore.Commerce.DynamicsRetail.TransactionService
{
    using Microsoft.Dynamics.Commerce.Runtime;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core.Models;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Services;
    using Newtonsoft.Json;
    using Sitecore.Commerce.DynamicsRetail.TransactionService.TransactionService;
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Globalization;
    using System.Linq;
    using System.Web;
    using Microsoft.Dynamics.Retail.Ecommerce.Sdk.Core;
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// Utilities class to be use by the different services.
    /// </summary>
    [ExcludeFromCodeCoverage]
    public static class Utilities
    {
        private static readonly double SessionTokenCookieMinutesUntilExpiration = TimeSpan.FromDays(30).TotalMinutes;

        /// <summary>
        /// Represent the url of an inactive link.
        /// </summary>
        public static readonly string InactiveLinkUrl = "#";

        /// <summary>
        /// Gets the cart identifier from request cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <param name="cartType">Type of the cart.</param>
        /// <returns></returns>
        /// <exception cref="System.InvalidOperationException">Only a shopping or checkout cart type is supported.</exception>
        public static string GetCartIdFromRequestCookie(SessionType sessionType, CartType cartType)
        {
            if (cartType == CartType.Checkout)
            {
                return GetCheckoutCartIdFromCookie(sessionType);
            }
            
            if (cartType == CartType.Shopping)
            {
                return GetShoppingCartIdFromCookie(sessionType);
            }
            
            string message = string.Format(CultureInfo.InvariantCulture, "CartType '{0}' not recognized.", cartType);
            throw new InvalidOperationException(message);
        }

        /// <summary>
        /// Gets the name of the cart token cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <param name="cartType">Type of the cart.</param>
        /// <returns>Name of the cookie that stores the cart identifier.</returns>
        /// <exception cref="System.InvalidOperationException"> Only specific cart types and session types are supported.
        /// </exception>
        public static string GetCartTokenCookieName(SessionType sessionType, CartType cartType)
        {
            string cookieName;
            string message;

            switch (sessionType)
            {
                case SessionType.Anonymous:
                    switch (cartType)
                    {
                        case CartType.Shopping:
                            cookieName = CookieConstants.AnonymousShoppingCartToken;
                            break;
                        case CartType.Checkout:
                            cookieName = CookieConstants.AnonymousCheckoutCartToken;
                            break;
                        default:
                            message = string.Format(CultureInfo.InvariantCulture, "CartType '{0}' is invalid.", cartType);
                            throw new InvalidOperationException(message);
                    }
                    break;

                case SessionType.SignedIn:
                    switch (cartType)
                    {
                        case CartType.Shopping:
                            cookieName = CookieConstants.SignedInShoppingCartToken;
                            break;
                        case CartType.Checkout:
                            cookieName = CookieConstants.SignedInCheckoutCartToken;
                            break;
                        default:
                            message = string.Format(CultureInfo.InvariantCulture, "CartType '{0}' is invalid.", cartType);
                            throw new InvalidOperationException(message);
                    }
                    break;

                default: message = string.Format(CultureInfo.InvariantCulture, "SessionType '{0}' not recognized.", sessionType);
                    throw new InvalidOperationException(message);
            }

            return cookieName;
        }

        /// <summary>
        /// Gets the shopping cart identifier based on the current session type.
        /// </summary>
        /// <returns>The shopping cart identifier that corresponds to the current session.</returns>
        public static string GetShoppingCartId()
        {
            SessionType sessionType = GetSessionType();
            string shoppingCartId = GetShoppingCartIdFromCookie(sessionType);
            return shoppingCartId;
        }

        /// <summary>
        /// Sets the cart identifier in response cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <param name="cartType">Type of the cart.</param>
        /// <param name="cartId">The cart identifier.</param>
        /// <exception cref="System.InvalidOperationException">Only a shopping or checkout cart type is supported.</exception>
        public static void SetCartIdInResponseCookie(SessionType sessionType, CartType cartType, string cartId)
        {
            if (cartType == CartType.Checkout)
            {
                SetCheckoutCartIdInCookie(sessionType, cartId);
            }
            else if (cartType == CartType.Shopping)
            {
                SetShoppingCartIdInCookie(sessionType, cartId);
            }
            else
            {
                string message = string.Format(CultureInfo.InvariantCulture, "CartType '{0}' not recognized.", cartType);
                throw new InvalidOperationException(message);
            }
        }

        /// <summary>
        /// Gets the checkout cart identifier based on the current session type.
        /// </summary>
        /// <returns>The checkout cart identifier that corresponds to the current session.</returns>
        public static string GetCheckoutCartId()
        {
            SessionType sessionType = GetSessionType();
            string checkoutCartId = GetCheckoutCartIdFromCookie(sessionType);
            return checkoutCartId;
        }

        /// <summary>
        /// Clears the identifiers for both the shopping cart and checkout cart from the response cookies.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        public static void ClearCartIdResponseCookies(SessionType sessionType)
        {
            ClearCheckoutCartIdResponseCookies(sessionType);
            ClearShoppingCartIdResponseCookies(sessionType);
        }

        /// <summary>
        /// Clears the checkout cart identifier in cookies.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        public static void ClearCheckoutCartIdResponseCookies(SessionType sessionType)
        {
            switch (sessionType)
            {
                case SessionType.Anonymous:
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.AnonymousCheckoutCartToken);
                    break;
                case SessionType.SignedIn:
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.SignedInCheckoutCartToken);
                    break;
                default:
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.AnonymousCheckoutCartToken);
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.SignedInCheckoutCartToken);
                    break;
            }
        }

        /// <summary>
        /// Clears the shopping cart identifier in cookies.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        public static void ClearShoppingCartIdResponseCookies(SessionType sessionType)
        {
            switch (sessionType)
            {
                case SessionType.Anonymous:
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.AnonymousShoppingCartToken);
                    break;
                case SessionType.SignedIn:
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.SignedInShoppingCartToken);
                    break;
                default:
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.AnonymousShoppingCartToken);
                    CookieManager.DeleteCookie(HttpContext.Current, CookieConstants.SignedInShoppingCartToken);
                    break;
            }
        }

        /// <summary>
        /// To the currency string.
        /// </summary>
        /// <param name="amount">The amount.</param>
        /// <returns></returns>
        public static string ToCurrencyString(this Decimal amount)
        {
            return ToCurrencyString(amount, CultureInfo.InvariantCulture.Name);
        }

        /// <summary>
        /// To the currency string.
        /// </summary>
        /// <param name="amount">The amount.</param>
        /// <param name="cultureName">Name of the culture.</param>
        /// <returns></returns>
        public static string ToCurrencyString(this Decimal amount, string cultureName)
        {
            string currencyStringTemplate = GetChannelCurrencyStringTemplate(cultureName);
            CultureInfo cultureInfo = !string.IsNullOrWhiteSpace(cultureName) ? new CultureInfo(cultureName) : CultureInfo.InvariantCulture;
            string str = amount.ToString("N2", cultureInfo);
            return string.Format(cultureInfo, currencyStringTemplate, str);
        }

        /// <summary>
        /// Gets the channel currency string template.
        /// </summary>
        /// <param name="cultureName">Name of the culture.</param>
        /// <returns></returns>
        public static string GetChannelCurrencyStringTemplate(string cultureName)
        {
            CultureInfo cultureInfo = !string.IsNullOrWhiteSpace(cultureName) ? new CultureInfo(cultureName) : CultureInfo.InvariantCulture;
            string channelCurrencySymbol = CrtUtilities.GetChannelCurrencySymbol();
            return cultureInfo.NumberFormat.CurrencyPositivePattern % 2 != 0 ? "{0}" + channelCurrencySymbol : channelCurrencySymbol + "{0}";
        }

        /// <summary>
        /// Sets the checkout cart identifier in cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <param name="checkoutCartId">The checkout cart identifier.</param>
        internal static void SetCheckoutCartIdInCookie(SessionType sessionType, string checkoutCartId)
        {
            const CartType cartType = CartType.Checkout;
            string cookieName = GetCartTokenCookieName(sessionType, cartType);

            CookieManager.SetResponseCookieValue(HttpContext.Current, cookieName, checkoutCartId, DateTime.Now.AddMinutes(SessionTokenCookieMinutesUntilExpiration), false, true);
        }

        /// <summary>
        /// Sets the shopping cart identifier in cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <param name="shoppingCartId">The shopping cart identifier.</param>
        internal static void SetShoppingCartIdInCookie(SessionType sessionType, string shoppingCartId)
        {
            const CartType cartType = CartType.Shopping;
            string cookieName = GetCartTokenCookieName(sessionType, cartType);

            CookieManager.SetResponseCookieValue(HttpContext.Current, cookieName, shoppingCartId, DateTime.Now.AddMinutes(SessionTokenCookieMinutesUntilExpiration), isSecure: false, isHttpOnly: true);
        }
        
        /// <summary>
        /// Gets the shopping cart identifier from cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <returns>The shopping cart identifer.</returns>
        internal static string GetShoppingCartIdFromCookie(SessionType sessionType)
        {
            string cookieName = GetCartTokenCookieName(sessionType, CartType.Shopping);
            string shoppingCartId = CookieManager.GetRequestCookieValue(HttpContext.Current, cookieName);
            return shoppingCartId;
        }

        /// <summary>
        /// Gets the checkout cart identifier from cookie.
        /// </summary>
        /// <param name="sessionType">Type of the session.</param>
        /// <returns>The checkout cart identifier.</returns>
        internal static string GetCheckoutCartIdFromCookie(SessionType sessionType)
        {
            string cookieName = GetCartTokenCookieName(sessionType, CartType.Checkout);
            string checkoutCartId = CookieManager.GetRequestCookieValue(HttpContext.Current, cookieName);
            return checkoutCartId;
        }

        /// <summary>
        /// Gets the type of the current session.
        /// </summary>
        /// <returns>The type of the current session.</returns>
        internal static SessionType GetSessionType()
        {
            var customerId = GetCustomerIdFromCookie();
            var sessionType = SessionType.Anonymous;
            if (!string.IsNullOrEmpty(customerId))
            {
                sessionType = SessionType.SignedIn;
            }

            return sessionType;
        }

        /// <summary>
        /// Sets the customer identifier in cookie.
        /// </summary>
        /// <param name="customerId">The customer identifier.</param>
        internal static void SetCustomerIdInCookie(string customerId)
        {
            string cookieName = CookieConstants.CustomerIdToken;
            CookieManager.SetResponseCookieValue(HttpContext.Current, cookieName, customerId, DateTime.Now.AddMinutes(SessionTokenCookieMinutesUntilExpiration), isSecure: false, isHttpOnly: true);
        }

        /// <summary>
        /// Gets the customer identifier from cookie.
        /// </summary>
        /// <returns>The customer identifier.</returns>
        internal static string GetCustomerIdFromCookie()
        {
            string cookieName = CookieConstants.CustomerIdToken;
            string customerId = CookieManager.GetRequestCookieValue(HttpContext.Current, cookieName);
            
            return (string.IsNullOrWhiteSpace(customerId) ? string.Empty : customerId);
        }

        /// <summary>
        /// Clears the customer identifier from cookie.
        /// </summary>
        internal static void ClearCustomerIdFromCookie()
        {
            string cookieName = CookieConstants.CustomerIdToken;
            CookieManager.DeleteCookie(HttpContext.Current, cookieName);
        }

        /// <summary>
        /// Sets the culture Id in cookie.
        /// </summary>
        /// <param name="language">The culture Id.</param>
        internal static void SetCultureIdInCookie(string language)
        {
            string cookieName = CookieConstants.CultureIdToken;
            // Setting HttpOnly (last parameter) to false so that we can access the cookies from the client 
            // (eg. Resources.ts is using cookies to retrieve UICulture).
            CookieManager.SetResponseCookieValue(HttpContext.Current, cookieName, language, DateTime.Now.AddMinutes(SessionTokenCookieMinutesUntilExpiration), isSecure: false, isHttpOnly: false);
        }

        /// <summary>
        /// Gets the culture Id from cookie.
        /// </summary>
        /// <returns>The Culture Id.</returns>
        internal static string GetCultureIdFromCookie()
        {
            string cookieName = CookieConstants.CultureIdToken;
            string language = CookieManager.GetRequestCookieValue(HttpContext.Current, cookieName);

            return (string.IsNullOrWhiteSpace(language) ? string.Empty : language);
        }

        /// <summary>
        /// Clears the culture Id from cookie.
        /// </summary>
        internal static void ClearCultureIdFromCookie()
        {
            string cookieName = CookieConstants.CultureIdToken;
            CookieManager.DeleteCookie(HttpContext.Current, cookieName);
        }
 
        /// <summary>
        /// Executes the given service action and handles exceptions.
        /// </summary>
        /// <param name="response">Service response.</param>
        /// <param name="action">Service action.</param>
        internal static void ExecuteAndHandleExceptions(ServiceResponse response, Action action)
        {
            if (response == null || response.Errors == null)
            {
                return;
            }

            try
            {
                action();
            }
            catch (Exception ex)
            {
                response.Errors.AddRange(HandleException(ex));
            }
        }

        /// <summary>
        /// Handles the exception.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <returns>A collection of response errors</returns>
        public static IEnumerable<ResponseError> HandleException(Exception ex)
        {
            if (ex == null)
            {
                ex = new ArgumentNullException("ex", "Exception cannot be null.");
            }

            var responseErrors = new Collection<ResponseError>();
            Type exceptionType = ex.GetType();

            if (exceptionType == typeof(NotificationException))
            {
                responseErrors.AddRange(ProcessNotificationException((NotificationException)ex));
            }
            else if (exceptionType == typeof(DataValidationException))
            {
                responseErrors.AddRange(ProcessDataValidationException((DataValidationException)ex));
            }
            else if (exceptionType == typeof(PaymentException))
            {
                // For now we do not have any special handling for a PaymentException type. Hence, treating it like any other commerce runtime exception.
                responseErrors.AddRange(ProcessCommerceRuntimeException((CommerceRuntimeException)ex));
            }
            else if (exceptionType == typeof(CommerceRuntimeException))
            {
                responseErrors.AddRange(ProcessCommerceRuntimeException((CommerceRuntimeException)ex));
            }
            else if (exceptionType == typeof(AggregateException))
            {
                responseErrors.AddRange(ProcessAggregateException((AggregateException)ex));
            }
            else
            {
                responseErrors.AddRange(ProcessUnrecognizedException(ex));
            }

            responseErrors = GetUniqueErrorMessageResponseErrors(responseErrors);
            return responseErrors;
        }

        /// <summary>
        /// Loads the product details.
        /// </summary>
        /// <param name="cart">The cart.</param>
        internal static void LoadProductDetails(ShoppingCart cart)
        {
            foreach (var item in cart.Items)
            {
                var productInfo = JsonConvert.DeserializeObject<Dictionary<string, object>>(item.ProductDetails);
                object imageUrl;
                productInfo.TryGetValue("ImageUrl", out imageUrl);
                object productUrl;
                productInfo.TryGetValue("ProductUrl", out productUrl);

                item.Image = new ImageInfo() { Url = imageUrl as string, AltText = string.Empty };
                item.ProductUrl = productUrl as string;
            }
        }

        #region private methods

        private static bool IsCurrentUserSignedIn()
        {
            string customerId = GetCustomerIdFromCookie();
            return !string.IsNullOrWhiteSpace(customerId);
        }
        
        private static IEnumerable<ResponseError> ProcessNotificationException(NotificationException notificationException)
        {
            Collection<ResponseError> responseErrors;

            var innerExceptionType = notificationException.InnerException.GetType();

            if (innerExceptionType == typeof(DataValidationException))
            {
                responseErrors = ProcessDataValidationException((DataValidationException)notificationException.InnerException);
            }
            else if (innerExceptionType == typeof(ConfigurationException) || innerExceptionType == typeof(CommerceRuntimeException))
            {
                responseErrors = ProcessCommerceRuntimeException((CommerceRuntimeException)notificationException.InnerException);
            }
            else
            {
                responseErrors = ProcessUnrecognizedException(notificationException.InnerException);
            }

            return responseErrors;
        }

        private static Collection<ResponseError> ProcessCommerceRuntimeException(CommerceRuntimeException ex)
        {
            var responseErrors = new Collection<ResponseError>();
            var errorMessage = GetErrorMessage(ex.ErrorResourceId, typeof(CommerceRuntimeException));
            var responseError = new ResponseError
                                {
                                    ErrorCode = ex.ErrorResourceId,
                                    ErrorMessage = errorMessage,
                                    Source = ex.Source,
                                    StackTrace = ex.StackTrace,
                                    ExtendedErrorMessage = ex.InnerException != null 
                                        ? string.Concat(ex.InnerException.Source, " ------------ ", ex.InnerException.Message, " ------------ ", ex.InnerException.StackTrace)
                                        : string.Empty
                                };

            responseErrors.Add(responseError);

            return responseErrors;
        }

        private static IEnumerable<ResponseError> ProcessAggregateException(AggregateException aggregrateException)
        {
            var responseErrors = new Collection<ResponseError>();
            foreach (Exception innerException in aggregrateException.InnerExceptions)
            {
                Type exceptionType = innerException.GetType();
                Collection<ResponseError> responseErrorsFromInnerException = exceptionType == typeof(CommerceRuntimeException) 
                    ? ProcessCommerceRuntimeException((CommerceRuntimeException)innerException)
                    : ProcessUnrecognizedException(innerException);

                responseErrors.AddRange(responseErrorsFromInnerException);
            }

            return responseErrors;
        }
        
        private static Collection<ResponseError> ProcessUnrecognizedException(Exception ex)
        {
            var responseErrors = new Collection<ResponseError>();
            var responseError = new ResponseError
                                {
                                    ErrorCode = string.Empty,
                                    ErrorMessage = ex.Message,
                                    Source = ex.Source,
                                    StackTrace = ex.StackTrace,
                                    ExtendedErrorMessage = ex.InnerException != null
                                        ? string.Concat(ex.InnerException.Source, " ------------ ", ex.InnerException.Message, " ------------ ", ex.InnerException.StackTrace)
                                        : string.Empty
                                };

            responseErrors.Add(responseError);

            return responseErrors;
        }
        
        private static Collection<ResponseError> ProcessDataValidationException(DataValidationException exception)
        {
            var responseErrors = new Collection<ResponseError>();

            if (exception.ValidationResults.Count == 0)
            {
                ResponseError responseError = ProcessDataValidationExceptionHelper(exception.ErrorResourceId, exception);
                responseErrors.Add(responseError);
            }
            else
            {
                foreach (DataValidationFailure validationResult in exception.ValidationResults)
                {
                    string errorResourceId = validationResult.ErrorResourceId;
                    if (errorResourceId.Equals("Microsoft_Dynamics_Commerce_Runtime_InvalidShippingAddress"))
                    {
                        errorResourceId = GetSpecificResourceCodeForInvalidShippingAddressException(validationResult);
                    }

                    ResponseError responseError = ProcessDataValidationExceptionHelper(errorResourceId, exception);
                    responseErrors.Add(responseError);
                }
            }

            return responseErrors;
        }

        private static string GetSpecificResourceCodeForInvalidShippingAddressException(DataValidationFailure validationResult)
        {
            string memberName;
            if (validationResult.MemberNames.Count > 0)
            {
                memberName = validationResult.MemberNames.First();
            }
            else
            {
                return validationResult.ErrorResourceId;
            }

            string errorResourceId;
            switch (memberName)
            {
                case "STATEPROVINCEID":
                    errorResourceId = "Microsoft_Dynamics_Commerce_Runtime_InvalidState";
                    break;
                case "City":
                    errorResourceId = "Microsoft_Dynamics_Commerce_Runtime_InvalidCity";
                    break;
                case "ZIPPOSTALCODE":
                    errorResourceId = "Microsoft_Dynamics_Commerce_Runtime_InvalidZipCode";
                    break;
                default:
                    errorResourceId = validationResult.ErrorResourceId;
                    break;
            }

            return errorResourceId;
        }

        private static string GetErrorMessage(string errorResourceId, Type exceptionType)
        {
            string errorMessage;
            if (string.IsNullOrWhiteSpace(errorResourceId))
            {
                if (exceptionType == typeof(DataValidationException))
                {
                    errorMessage = ServiceResources.GenericValidationError;
                }
                else if (exceptionType == typeof(CommerceRuntimeException))
                {
                    errorMessage = ServiceResources.GenericCommerceError;
                }
                else
                {
                    errorMessage = ServiceResources.GenericErrorMessage;
                }
            }
            else
            {
                errorMessage = ServiceResources.ResourceManager.GetString(errorResourceId);
            }

            return errorMessage;
        }

        private static ResponseError ProcessDataValidationExceptionHelper(string errorResourceId, Exception exception)
        {
            string errorMessage = GetErrorMessage(errorResourceId, typeof(DataValidationException));
            var responseError = new ResponseError
                                {
                                    ErrorCode = errorResourceId,
                                    ErrorMessage = errorMessage,
                                    Source = exception.Source,
                                    StackTrace = exception.StackTrace,
                                    ExtendedErrorMessage = exception.InnerException != null
                                        ? string.Concat(exception.InnerException.Source, " ------------ ", exception.InnerException.Message, " ------------ ", exception.InnerException.StackTrace)
                                        : string.Empty
                                };

            return responseError;
        }
        
        private static Collection<ResponseError> GetUniqueErrorMessageResponseErrors(IEnumerable<ResponseError> responseErrors)
        {
            var uniqueResponseErrors = new Collection<ResponseError>();
            var errorMessageHashSet = new HashSet<string>();

            foreach (var responseError in responseErrors)
            {
                string errorMessage = responseError.ErrorMessage ?? string.Empty;
                if (errorMessageHashSet.Contains(errorMessage))
                {
                    continue;
                }

                uniqueResponseErrors.Add(responseError);
                errorMessageHashSet.Add(errorMessage);
            }

            return uniqueResponseErrors;
        }

        #endregion
    }
}